require 'spec_helper'

describe PuppetX::Eos::Radius do
  let(:eapi) { PuppetX::Eos::Eapi.new }
  let(:instance) { described_class.new(eapi) }

  describe '#getall' do
    context 'when all radius settings are the default' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_radius_default))
      end

      describe 'the return value' do
        subject { instance.getall }

        it { is_expected.to be_an Array }
        it 'has only one resource hash' do
          expect(subject.size).to eq(1)
        end
      end

      describe 'the single resource hash' do
        subject { instance.getall.first }

        it { is_expected.to include(name: 'settings') }
        it { is_expected.to include(enable: true) }
        it { is_expected.to_not include(:key) }
        it { is_expected.to_not include(:key_format) }
        it { is_expected.to include(timeout: 5) }
        it { is_expected.to include(retransmit_count: 3) }
      end
    end

    context 'when key, timeout, and retransmit are configured' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_radius_configured))
      end
      subject { instance.getall.first }

      it { is_expected.to include(name: 'settings') }
      it { is_expected.to include(enable: true) }
      it { is_expected.to include(key: '070E234F1F5B4A') }
      it { is_expected.to include(key_format: 7) }
      it { is_expected.to include(timeout: 50) }
      it { is_expected.to include(retransmit_count: 5) }
    end
  end

  describe '#servers' do
    context 'with radius servers configured' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_radius_servers))
      end

      describe 'the return value' do
        subject { instance.servers }

        it 'parses 6 results' do
          expect(subject.size).to eq(6)
        end
        it { is_expected.to be_an Array }
        it 'includes only Hashes as elements of the Array' do
          subject.each { |result| expect(result).to be_a Hash }
        end
        it 'includes hostname in every result' do
          subject.each { |result| expect(result).to include(:hostname) }
        end
        it 'includes auth_port in every result' do
          subject.each { |result| expect(result).to include(:auth_port) }
        end
        it 'includes acct_port in every result' do
          subject.each { |result| expect(result).to include(:acct_port) }
        end
      end
    end
  end

  describe '#server_groups' do
    before :each do
      allow(instance).to receive(:running_configuration)
        .and_return(fixture(:running_configuration_radius_server_groups))
    end

    describe 'the return value' do
      subject { instance.server_groups }

      it { is_expected.to be_an Array }
      it 'returns 3 server groups' do
        expect(subject.size).to eq 3
      end

      describe 'each value of the return Array' do
        it 'each value is a hash' do
          subject.each {|v| expect(v).to be_a Hash }
        end
        it 'each value has a name attribute' do
          subject.each {|v| expect(v).to include(name: /RAD-SV/) }
        end
        it 'each value has a servers attribute' do
          subject.each {|v| expect(v).to include(servers: Array) }
        end
      end

      describe 'each server attribute' do
        it 'is a hash' do
          subject.each do |group|
            group[:servers].each do |server|
              expect(server).to be_a Hash
            end
          end
        end
        it 'includes the hostname, auth_port, acct_port attributes' do
          subject.each do |group|
            group[:servers].each do |server|
              expect(server).to include(hostname: String,
                                        auth_port: Fixnum,
                                        acct_port: Fixnum)
            end
          end
        end
      end
    end
  end

  describe '#update_server_group' do
    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.update_server_group(opts) }

    context 'with no servers specified' do
      let :opts do
        { name: 'RAD-SV9', servers: [] }
      end

      it { is_expected.to eq(true) }

      it 're-creates the server group after clearing it' do
        expect(instance.api).to receive(:config)
          .with('no aaa group server radius RAD-SV9').ordered
        expect(instance.api).to receive(:config)
          .with(['aaa group server radius RAD-SV9']).ordered
        subject
      end
    end

    context 'with two servers specified' do
      let :opts do
        { name: 'RAD-SV9',
          servers: [
            { hostname: 'one' },
            { hostname: 'two', auth_port: 1234, acct_port: 5678 }
          ]
        }
      end

      it { is_expected.to eq(true) }
      it 'uses the default auth and acct ports when not provided' do
        expect(instance.api).to receive(:config)
          .with([/RAD-SV9/, 'server one auth-port 1812 acct-port 1813', String ])
        subject
      end
      it 'creates the group of servers using a single config call' do
        expect(instance.api).to receive(:config)
          .with([/RAD-SV9/, /server one/, /server two/])
        subject
      end
    end
  end

  describe '#remove_server_group' do
    let :opts do
      { name: 'RAD-SV9' }
    end

    subject { instance.remove_server_group(opts) }

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    it { is_expected.to eq true }
    it 'removes the group specified by name' do
      expect(instance.api).to receive(:config)
        .with('no aaa group server radius RAD-SV9')
      subject
    end
  end

  describe '#set_global_key' do
    let :opts do
      { key: '070E234F1F5B4A', key_format: 7 }
    end

    before :each do
      allow(instance.api).to receive(:config)
    end

    subject { instance.set_global_key(opts) }

    context 'when key_format is 7 (hashed)' do
      it 'uses Eapi#config to set the global key' do
        expect(instance.api).to receive(:config)
          .with('radius-server key 7 070E234F1F5B4A')
        subject
      end
    end

    context 'when key_format is not provided' do
      let :opts do
        { key: '070E234F1F5B4A' }
      end

      it 'assumes a default format of 7' do
        expect(instance.api).to receive(:config)
          .with('radius-server key 7 070E234F1F5B4A')
        subject
      end
    end

    context 'when key is not provided' do
      let :opts do
        { key_format: 7 }
      end

      it 'raises ArgumentError that key is required' do
        expect { subject }
          .to raise_error ArgumentError, /key option is required/
      end
    end

    context 'when the API call returns [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{}])
      end

      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end

  describe '#set_timeout' do
    let :opts do
      { timeout: 50 }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.set_timeout(opts) }

    context 'when timeout is not provided' do
      let :opts do
        {}
      end

      it 'raises ArgumentError' do
        expect { subject }
          .to raise_error ArgumentError, /timeout option is required/
      end
    end

    context 'when timeout is provided' do
      it 'calls Eapi#config to configure radius-server timeout' do
        expect(instance.api).to receive(:config)
          .with('radius-server timeout 50')
          .and_return([{}])
        subject
      end
    end

    context 'when the API call returns [{}]' do
      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end

  describe '#set_retransmit_count' do
    let :opts do
      { retransmit_count: 5 }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.set_retransmit_count(opts) }

    context 'when retransmit_count is not provided' do
      let :opts do
        {}
      end

      it 'raises ArgumentError' do
        expect { subject }
          .to raise_error ArgumentError, /retransmit_count option is required/
      end
    end

    context 'when retransmit_count is provided' do
      it 'calls Eapi#config to configure radius-server retransmit' do
        expect(instance.api).to receive(:config)
          .with('radius-server retransmit 5')
          .and_return([{}])
        subject
      end
    end

    context 'when the API call returns [{}]' do
      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end

  describe '#update_server' do
    let :opts do
      { hostname: '10.11.12.13' }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.update_server(opts) }

    context 'when only hostname is provided' do
      it 'calls Eapi#config to configure radius-server host' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, auth_port are provided' do
      let :opts do
        { hostname: '10.11.12.13', auth_port: 1024 }
      end

      it 'calls Eapi#config to configure radius-server host' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13 auth-port 1024')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, acct_port are provided' do
      let :opts do
        { hostname: '10.11.12.13', acct_port: 2048 }
      end

      it 'calls Eapi#config to configure radius-server host' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13 acct-port 2048')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, auth_port, acct_port are provided' do
      let :opts do
        { hostname: '10.11.12.13', auth_port: 1024, acct_port: 2048 }
      end

      it 'calls Eapi#config to configure radius-server host' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13 auth-port 1024 acct-port 2048')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, timeout are provided' do
      let :opts do
        { hostname: '10.11.12.13', timeout: 10 }
      end

      it 'calls Eapi#config to configure radius-server host with timeout' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13 timeout 10')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, auth_port, acct_port timeout are provided' do
      let :opts do
        {
          hostname: '10.11.12.13',
          auth_port: 1024,
          acct_port: 2048,
          timeout: 10
        }
      end

      it 'calls Eapi#config with auth-port first' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13 auth-port 1024 '\
                'acct-port 2048 timeout 10')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, key and no key_format are provided' do
      let :opts do
        { hostname: '10.11.12.13', key: 'abc123' }
      end

      it 'calls Eapi#config with key format 7 as the default' do
        expect(instance.api).to receive(:config)
          .with('radius-server host 10.11.12.13 key 7 abc123')
          .and_return([{}])
        subject
      end
    end

    context 'when the API call returns [{}]' do
      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end

  describe '#remove_server' do
    let :opts do
      { hostname: '10.11.12.13' }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.remove_server(opts) }

    context 'when hostname is provided' do
      it 'calls Eapi#config to configure no radius-server host ...' do
        expect(instance.api).to receive(:config)
          .with('no radius-server host 10.11.12.13')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname, auth_port, acct_port are provided' do
      let :opts do
        { hostname: '10.11.12.13', auth_port: 1024, acct_port: 2048 }
      end

      it 'calls Eapi#config with host, auth-port, acct-port in order' do
        expect(instance.api).to receive(:config)
          .with('no radius-server host 10.11.12.13 auth-port 1024 acct-port 2048')
          .and_return([{}])
        subject
      end
    end

    context 'when the API call returns [{}]' do
      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end
end
