require 'spec_helper'

describe PuppetX::Eos::Tacacs do
  let(:eapi) { PuppetX::Eos::Eapi.new }
  let(:instance) { described_class.new(eapi) }

  describe '#getall' do
    subject { instance.getall }

    context 'when all tacacs settings are the default' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_default))
      end

      describe 'the return value' do
        it { is_expected.to be_an Array }
        it 'has only one resource hash' do
          expect(subject.size).to eq(1)
        end
      end

      describe 'the single resource hash' do
        subject { instance.getall.first }

        it { is_expected.to include(name: 'settings') }
        it { is_expected.to include(enable: true) }
        it { is_expected.to_not include(:key) }
        it { is_expected.to_not include(:key_format) }
        it { is_expected.to include(timeout: 5) }
      end
    end

    context 'when a global timeout and key are configured' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_configured))
      end

      describe 'the return value' do
        it { is_expected.to be_an Array }
        it 'has only one resource hash' do
          expect(subject.size).to eq(1)
        end
      end

      describe 'the single resource hash' do
        subject { instance.getall.first }

        it { is_expected.to include(name: 'settings') }
        it { is_expected.to include(enable: true) }
        it { is_expected.to include(key: '070E234F1F5B4A') }
        it { is_expected.to include(key_format: 7) }
        it { is_expected.to include(timeout: 7) }
      end
    end
  end

  describe '#set_global_key' do
    let :opts do
      { key: '070E234F1F5B4A', key_format: 7 }
    end

    before :each do
      allow(instance.api).to receive(:config)
    end

    subject { instance.set_global_key(opts) }

    context 'when key_format is 7 (hashed)' do
      it 'uses Eapi#config to set the global key' do
        expect(instance.api).to receive(:config)
          .with('tacacs-server key 7 070E234F1F5B4A')
        subject
      end
    end

    context 'when key_format is not provided' do
      let :opts do
        { key: '070E234F1F5B4A' }
      end

      it 'assumes a default format of 7' do
        expect(instance.api).to receive(:config)
          .with('tacacs-server key 7 070E234F1F5B4A')
        subject
      end
    end

    context 'when key is not provided' do
      let :opts do
        { key_format: 7 }
      end

      it 'raises ArgumentError that key is required' do
        expect { subject }
          .to raise_error ArgumentError, /key option is required/
      end
    end

    context 'when the API call returns [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{}])
      end

      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end

  describe '#set_timeout' do
    let :opts do
      { timeout: 50 }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.set_timeout(opts) }

    context 'when timeout is not provided' do
      let :opts do
        {}
      end

      it 'raises ArgumentError' do
        expect { subject }
          .to raise_error ArgumentError, /timeout option is required/
      end
    end

    context 'when timeout is provided' do
      it 'calls Eapi#config to configure tacacs-server timeout' do
        expect(instance.api).to receive(:config)
          .with('tacacs-server timeout 50')
          .and_return([{}])
        subject
      end
    end

    context 'when the API call returns [{}]' do
      it { is_expected.to eq true }
    end

    context 'when the API call does not return [{}]' do
      before :each do
        allow(instance.api).to receive(:config)
          .and_return([{'errors' => ['blarg']}])
      end

      it { is_expected.to eq false }
    end
  end

  describe '#servers' do
    subject { instance.servers }

    context 'when there are tacacs servers defined' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_servers))
      end
      it { is_expected.to be_an Array }
      it 'parses 4 results' do
        expect(subject.size).to eq(4)
      end
      it 'includes only Hashes as elements of the Array' do
        subject.each { |result| expect(result).to be_a Hash }
      end
      it 'includes hostname in every result' do
        subject.each { |result| expect(result).to include(:hostname) }
      end
      it 'includes port in every result' do
        subject.each { |result| expect(result).to include(:port) }
      end
      it 'includes one entry with multiplex true' do
        expected = {
          port: 4949,
          hostname: "1.2.3.4",
          timeout: 6,
          key_format: 7,
          key: "06070D221D1C5A",
          multiplex: true
        }
        expect(subject).to include(expected)
      end
    end

    context 'when there are no tacacs servers defined' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_default))
      end
      it { is_expected.to be_an Array }
      it { is_expected.to be_empty }
    end
  end

  describe '#update_server' do
    subject { instance.update_server(opts) }

    let :opts do
      { hostname: '10.11.12.13' }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    context 'when only hostname is provided' do
      it 'calls Eapi#config to configure tacacs-server host' do
        expect(instance.api).to receive(:config)
          .with('tacacs-server host 10.11.12.13')
          .and_return([{}])
        subject
      end
    end

    context 'when configuring a single-connection server' do
      let :opts do
        { hostname: '10.11.12.13', multiplex: true }
      end
      it 'calls Eapi#config to with single-connection' do
        expect(instance.api).to receive(:config)
          .with('tacacs-server host 10.11.12.13 single-connection')
          .and_return([{}])
        subject
      end
    end
  end

  describe '#remove_server' do
    subject { instance.remove_server(opts) }

    let :opts do
      { hostname: '10.11.12.13' }
    end

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    context 'when only hostname is provided' do
      it 'calls Eapi#config to configure no tacacs-server host' do
        expect(instance.api).to receive(:config)
          .with('no tacacs-server host 10.11.12.13')
          .and_return([{}])
        subject
      end
    end

    context 'when hostname and port are provided' do
      let :opts do
        { hostname: '10.11.12.13', port: 4949 }
      end
      it 'calls Eapi#config to configure no tacacs-server host port 4949' do
        expect(instance.api).to receive(:config)
          .with('no tacacs-server host 10.11.12.13 port 4949')
          .and_return([{}])
        subject
      end
    end
  end

  describe '#server_groups' do
    context 'with two server groups defined' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_groups))
      end

      describe 'the return value' do
        subject { instance.server_groups }

        it { is_expected.to be_an Array }
        it 'returns 2 server groups' do
          expect(subject.size).to eq 2
        end

        describe 'each value of the return Array' do
          it 'each value is a hash' do
            subject.each {|v| expect(v).to be_a Hash }
          end
          it 'each value has a name attribute' do
            subject.each {|v| expect(v).to include(name: /TAC-GR/) }
          end
          it 'each value has a servers attribute' do
            subject.each {|v| expect(v).to include(servers: Array) }
          end
        end

        describe 'each server attribute' do
          it 'is a hash' do
            subject.each do |group|
              group[:servers].each do |server|
                expect(server).to be_a Hash
              end
            end
          end
          it 'includes the hostname, port attributes' do
            subject.each do |group|
              group[:servers].each do |server|
                expect(server).to include(hostname: String, port: Fixnum)
              end
            end
          end
        end
      end
    end

    context 'with a server group with no servers' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_groups_3))
      end

      describe 'the return value' do
        subject { instance.server_groups }

        it { is_expected.to be_an Array }
        it 'returns 3 server groups' do
          expect(subject.size).to eq 3
        end

        describe 'each value of the return Array' do
          it 'each value is a hash' do
            subject.each {|v| expect(v).to be_a Hash }
          end
          it 'each value has a name attribute' do
            subject.each {|v| expect(v).to include(name: /TAC-GR/) }
          end
          it 'each value has a servers attribute' do
            subject.each {|v| expect(v).to include(servers: Array) }
          end
        end

        describe 'each server attribute' do
          it 'is a hash' do
            subject.each do |group|
              group[:servers].each do |server|
                expect(server).to be_a Hash
              end
            end
          end
          it 'includes the hostname, port attributes' do
            subject.each do |group|
              group[:servers].each do |server|
                expect(server).to include(hostname: String, port: Fixnum)
              end
            end
          end
        end
      end
    end

    context 'with no server groups defined' do
      before :each do
        allow(instance).to receive(:running_configuration)
          .and_return(fixture(:running_configuration_tacacs_default))
      end

      describe 'the return value' do
        subject { instance.server_groups }

        it { is_expected.to be_an Array }
        it 'returns 0 server groups' do
          expect(subject.size).to eq 0
        end
      end
    end
  end

  describe '#update_server_group' do
    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    subject { instance.update_server_group(opts) }

    context 'with no servers specified' do
      let :opts do
        { name: 'TAC-GR-EMPTY', servers: [] }
      end

      it { is_expected.to eq(true) }

      it 're-creates the server group after clearing it' do
        expect(instance.api).to receive(:config)
          .with('no aaa group server tacacs+ TAC-GR-EMPTY').ordered
        expect(instance.api).to receive(:config)
          .with(['aaa group server tacacs+ TAC-GR-EMPTY']).ordered
        subject
      end
    end

    context 'with two servers specified' do
      let :opts do
        { name: 'TAC-GR',
          servers: [
            { hostname: 'one' },
            { hostname: 'two', port: 50 }
          ]
        }
      end

      it { is_expected.to eq(true) }
      it 'uses the default port when not provided' do
        expect(instance.api).to receive(:config)
          .with([/TAC-GR/, 'server one port 49', String ])
        subject
      end
      it 'creates the group of servers using a single config call' do
        expect(instance.api).to receive(:config)
          .with([/TAC-GR/, /server one/, /server two/])
        subject
      end
    end
  end

  describe '#remove_server_group' do
    let :opts do
      { name: 'TAC-GR' }
    end

    subject { instance.remove_server_group(opts) }

    before :each do
      allow(instance.api).to receive(:config).and_return([{}])
    end

    it { is_expected.to eq true }
    it 'removes the group specified by name' do
      expect(instance.api).to receive(:config)
        .with('no aaa group server tacacs+ TAC-GR')
      subject
    end
  end
end
